

// --------------------------------------------------------- 
//	LD48 #5 (I think)
//	
//	Theme: 'Light+Darkness'
//	By Dave Reed (bluescreen)
//
//	Entities.cpp
// 
//  Players, enemies, bullets, etc
// 
// --------------------------------------------------------- 

#include "ld48.h"


void CGame::InitEntities(void)
{
	for(int n=0;n<MAX_ENTITIES;n++)
	{		
		pEntities[n]=NULL;		
	}
}

void CGame::AddEntity(Entity *pE)
{
	for(int n=0;n<MAX_ENTITIES;n++)
	{
		if (pEntities[n]==NULL)		
		{
			pEntities[n]=pE;
			pE->bAlive=true;
			return;		}
	}
}

void CGame::ResetEntities(void)
{
	for(int n=0;n<MAX_ENTITIES;n++)
	{
		if (pEntities[n])
		{
			delete pEntities[n];
			pEntities[n]=NULL;
		}
	}
}



void CGame::UpdateEntities(void)
{
	for(int n=0;n<MAX_ENTITIES;n++)
	{
		if (pEntities[n])
		{	
			if (pEntities[n]->bAlive)
			{
				pEntities[n]->Update(this);

				for(int i=0;i<MAX_ENTITIES;i++)
				{
					if (pEntities[i])
					{
						Entity *p1=pEntities[i];
						Entity *p2=pEntities[n];

						float fD= (p1->fRadius*p1->fRadius) + (p2->fRadius*p2->fRadius);
						
						float fX=p1->vPos.x-p2->vPos.x;
						float fY=p1->vPos.y-p2->vPos.y;

						if ((fX*fX+fY*fY)<fD)
						{
							p1->HandleCollision(this,p2);
						}
		
					}
				}
			}

			if (!pEntities[n]->bAlive)
			{
				if (pEntities[n]->nEntityType!=ENTITY_PLAYER)
				{
					delete pEntities[n];
					pEntities[n]=0;
				}
			}

		}
	}
}

void CGame::RenderEntities(void)
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	//glAlphaFunc(GL_GREATER,0.5f);
	//glEnable(GL_ALPHA_TEST);

	for(int n=0;n<MAX_ENTITIES;n++)
	{
		if (pEntities[n])
		{	
			if (pEntities[n]->bAlive)
			{
				pEntities[n]->Render();
			}
		}
	}

	//glDisable(GL_ALPHA_TEST);
}

void CGame::RenderEntityLights(void)
{
	for(int n=0;n<MAX_ENTITIES;n++)
	{
		if (pEntities[n])
		{	
			if (pEntities[n]->bAlive)
			{
				pEntities[n]->RenderLights(this);
			}
		}
	}
}



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// PLAYER
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

Player::Player(Sprite *pSprMain,Sprite *pSprTurret, float fX,float fY)
{
	this->pSprMain=pSprMain;
	this->pSprTurret=pSprTurret;
	vPos.x=fX;
	vPos.y=fY;
	vVel.x=0;
	vVel.y=0;
	fAngle=PI/2;
	fRotVel=fSpeed=fThrust=0;
	nEntityType=ENTITY_PLAYER;
}


void Player::Render(void)
{
	static int nFrame=0;
	nFrame++;
	if (nFrame>127) nFrame=0;
	
	pSprMain[nFrame>>1].Blit(vPos.x,vPos.y,0);
	pSprTurret->Blit(vPos.x,vPos.y,fAngle-(PI/2));
}


void Player::RenderLights(CGame *pGame)
{
	
	// spot
	LIGHT Light;	
	Light.vPos=vPos;	
	Light.fR=0.8f;
	Light.fG=0.8f;
	Light.fB=0.8f;
	Light.nSubDivisions=1024;
	Light.fRange=400;	
	Light.nTexID=TEX_SPOTLIGHT;
	Light.fAngle=fAngle;
	pGame->RenderLight(&Light);

	// area	
	Light.vPos=vPos;	
	Light.fR=0.3f;
	Light.fG=0.3f;
	Light.fB=0.2f;
	Light.nSubDivisions=128;
	Light.fRange=80;	
	Light.nTexID=TEX_POINTLIGHT;
	Light.fAngle=0;
	pGame->RenderLight(&Light);

	// thrust
	if (fThrust>0)
	{
		fThrust-=0.02f;
		LIGHT Light;	
		Light.vPos=vPos;	
		Light.fR=1.5f*fThrust;
		Light.fG=1.0f*fThrust;
		Light.fB=0.5f*fThrust;
		Light.nSubDivisions=256;
		Light.fRange=150;	
		Light.nTexID=TEX_SPOTLIGHT;
		Light.fAngle=PI;
		pGame->RenderLight(&Light);
	}
	
}

void Player::Update(CGame *pGame)
{
	if (KEY_State(KEY_LEFT))  vVel.x-=0.15f;
	if (KEY_State(KEY_RIGHT)) vVel.x+=0.15f;
	
	if (KEY_State(KEY_UP))	 
	{
		if (bOnGround)
		{
			SFX_Play(SFX_JUMP);
			vVel.y=-6;
			fThrust=1;
		}
	}
	

	if (KEY_State(KEY_TLEFT))  fAngle-=0.032f;
	if (KEY_State(KEY_TRIGHT)) fAngle+=0.032f;
	

	if (KEY_Pressed(KEY_FIRE))
	{
		float fShotSpeed=8;
		SFX_Play(SFX_SHOOT);
		Bullet *pB= new Bullet(&pGame->sprBullet,0,fAngle,vPos.x,vPos.y,
							(vVel.x)-(fShotSpeed*-sin(fAngle)),
							(vVel.y)- (fShotSpeed*cos(fAngle)));

		pGame->AddEntity(pB);	
	}

	vVel.x*=0.95f;	
	vVel.y+=0.16f; // gravity
	fRotVel*=0.95f;

	
	fAngle+=fRotVel;

	if (!bOnGround)	
		nAirborneFrames++;	
	else
		nAirborneFrames=0;

	bOnGround=false;

	// Collisions
	float fOld=vPos.x;
	bool  bHit=false;
	vPos.x+=vVel.x;
	if (IsTileSolid(GetTile(vPos.x+12,vPos.y-12))) bHit=true;
	if (IsTileSolid(GetTile(vPos.x-12,vPos.y-12))) bHit=true;
	if (IsTileSolid(GetTile(vPos.x+12,vPos.y+12))) bHit=true;
	if (IsTileSolid(GetTile(vPos.x-12,vPos.y+12))) bHit=true;
	if (bHit)
	{
		vPos.x=fOld;		
	}

	fOld=vPos.y;
	bHit=false;
	vPos.y+=vVel.y;
	if (IsTileSolid(GetTile(vPos.x+12,vPos.y-12))) {bHit=true; if (vVel.y<0) vVel.y=0;}
	if (IsTileSolid(GetTile(vPos.x-12,vPos.y-12))) {bHit=true; if (vVel.y<0) vVel.y=0;}
	if (IsTileSolid(GetTile(vPos.x+12,vPos.y+12))) {bHit=true; bOnGround=true; if (vVel.y>0) vVel.y=0;}
	if (IsTileSolid(GetTile(vPos.x-12,vPos.y+12))) {bHit=true; bOnGround=true; if (vVel.y>0) vVel.y=0;}
	if (bHit)
	{
		if (nAirborneFrames>8 && bOnGround) SFX_Play(SFX_LAND);
		vPos.y=fOld;		
	}
}



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// BULLETS
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

Bullet::Bullet(Sprite *pSpr,int nType,float fAngle,float fX,float fY, float fXV,float fYV)
{
	vPos.x=fX;
	vPos.y=fY;
	vVel.x=fXV;
	vVel.y=fYV;
	this->fAngle=fAngle;
	this->pSpr=pSpr;
	fRadius=4;
	nEntityType=ENTITY_BULLET;
}

void Bullet::Render()
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);
	pSpr->Blit(vPos.x,vPos.y,fAngle);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
}

void Bullet::RenderLights(CGame *pGame)
{
	// area	
	LIGHT Light;
	Light.vPos=vPos;	
	Light.fR=0.6f;
	Light.fG=0.4f;
	Light.fB=0.2f;
	Light.nSubDivisions=128;
	Light.fRange=90+rand()&63;	
	Light.nTexID=TEX_POINTLIGHT;
	Light.fAngle=fAngle;
	pGame->RenderLight(&Light);
}

void Bullet::Update(CGame *pGame)
{
	vPos.x+=vVel.x;
	vPos.y+=vVel.y;	

	int n=GetTile(vPos.x,vPos.y);
	if (IsTileSolid(n))
	{
		if (n==2)
		{
			ClearTile(vPos.x,vPos.y);

			// particles
			SFX_Play(SFX_IMPACT);
			Particles *pFX= new Particles(&pGame->sprExplode,64,PARTICLES_SPARKS2,vPos.x,vPos.y,0,0);
			pGame->AddEntity(pFX);
		}
		else
		{
			vPos.x-=vVel.x;
			vPos.y-=vVel.y;	
			SFX_Play(SFX_IMPACT);
			Particles *pFX= new Particles(&pGame->sprExplode,10,PARTICLES_IMPACT,vPos.x,vPos.y,0,0);
			pGame->AddEntity(pFX);
		}

		bAlive=false;
	}
}


// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// LIGHTS
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

LightSource::LightSource(Sprite *pSpr, int nType,float fX,float fY)
{
	vPos.x=fX;
	vPos.y=fY;	
	this->pSpr=pSpr;
	this->nType=nType;
	this->fRadius=10;
	nEntityType=ENTITY_OTHER;
}

void LightSource::Update(CGame *pGame) {};	
void LightSource::Render() 
{
	if(pSpr)
	{
		pSpr->Blit(vPos.x,vPos.y,PI+this->fAngle);
	}
};

void LightSource::RenderLights(CGame *pGame) 
{
	if (nType==0)
	{	
		LIGHT Light;
		Light.vPos=vPos;	
		Light.fR=0.7f;
		Light.fG=0.7f;
		Light.fB=0.2f;
		Light.nSubDivisions=128;
		Light.fRange=300;	
		Light.nTexID=TEX_SPOTLIGHT;
		Light.fAngle=fAngle=PI+(sin(vPos.x+gfGameTimer*2.5f)*0.2f);
		pGame->RenderLight(&Light);
	}

	if (nType==1)
	{	
		LIGHT Light;
		Light.vPos=vPos;	
		Light.fR=1.0f;
		Light.fG=0.0f;
		Light.fB=0.0f;
		Light.nSubDivisions=128;
		Light.fRange=150;	
		Light.nTexID=TEX_SPINLIGHT;
		Light.fAngle=fAngle=(vPos.x+gfGameTimer*1.5f);
		pGame->RenderLight(&Light);
	}
};


void LightSource::HandleCollision(CGame *pGame,Entity *pEntity)
{
	if (pEntity->nEntityType==ENTITY_BULLET)
	{
		Bullet *pBullet = (Bullet *)pEntity;
		pBullet->bAlive=false;
		bAlive=false;

		pGame->nScore+=50;

		SFX_Play(SFX_IMPACT);
		Particles *pFX= new Particles(&pGame->sprExplode,64,PARTICLES_SPARKS,vPos.x,vPos.y,0,0);
		pGame->AddEntity(pFX);
	}	
};



// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// ENEMIES
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

Enemy::Enemy(Sprite *pSpr,int nType, float fX,float fY)
{
	fRadius=12;
	nHP=3;
	this->pSpr=pSpr;
	nFrame=rand()&63;	
	vPos.x=fX;
	vPos.y=fY;
	vVel.x=1;

	if (nType==ENEMY_ROLLER) vVel.x=2;

	vVel.y=0.5f;
	fAngle=0;
	nEntityType=ENTITY_ENEMY;
	this->nType=nType;
}

void Enemy::Update(CGame *pGame)
{

	if (nType==ENEMY_FLOATER)
	{
		VEC2 vP;
		vP.x=pGame->pPlr->vPos.x-vPos.x;
		vP.y=pGame->pPlr->vPos.y-vPos.y;

		if ((vP.x*vP.x+vP.y*vP.y)<(260*260) && pGame->pPlr->bAlive)
		{
			if (fGlow<=0)
			{
				//vVelOld=vVel;
			}

			// check line of sight
			float fL=sqrt(vP.x*vP.x+vP.y*vP.y);
			vP.x/=fL;
			vP.y/=fL;


			float fC=CastRay(vPos,  vP, 260);
			if (fC>=(fL*0.9f))
			{
				fGlow+=0.1f;
				if (fGlow>1)
				{
					fGlow=1;					
					

					if (fL<64) fL=64;
					vVel.x+=(vP.x*(30/fL));
					vVel.y+=(vP.y*(30/fL));

					float fSpd=sqrt((vVel.x*vVel.x)+(vVel.y*vVel.y));
					if (fSpd>2)
					{
						vVel.x=(vVel.x*2.0f)/fSpd;
						vVel.y=(vVel.y*2.0f)/fSpd;
					}			
				}
			}
			else
			{
				fGlow-=0.05f;
				if (fGlow<0) fGlow=0;
			}
		}
		else
		{
			fGlow-=0.1f;
			if (fGlow<0) fGlow=0;
		}
	

		bool bHit=false;
		float fOld=vPos.x;
		vPos.x+=vVel.x;
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y-12))) bHit=true;
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y-12))) bHit=true;
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y+12))) bHit=true;
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y+12))) bHit=true;
		if (bHit)
		{
			vPos.x=fOld;		
			vVel.x=-vVel.x;
		}

		fOld=vPos.y;
		bHit=false;
		vPos.y+=vVel.y;
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y-12))) {bHit=true;}
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y-12))) {bHit=true;}
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y+12))) {bHit=true;}
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y+12))) {bHit=true;}
		if (bHit)
		{
			vPos.y=fOld;		
			vVel.y=-vVel.y;
		}
	}

	if (nType==ENEMY_CRAWLER)
	{
		vVel.y+=0.1f; // gravity;

		bool bHit=false;
		float fOld=vPos.x;
		vPos.x+=vVel.x;
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y-12))) bHit=true;
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y-12))) bHit=true;
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y+12))) bHit=true;
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y+12))) bHit=true;
		if (bHit)
		{
			vPos.x=fOld;		
			vVel.x=-vVel.x;
		}

		fOld=vPos.y;
		bHit=false;
		vPos.y+=vVel.y;
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y-12))) {bHit=true;}
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y-12))) {bHit=true;}
		if (IsTileSolid(GetTile(vPos.x+12,vPos.y+12))) {bHit=true;}
		if (IsTileSolid(GetTile(vPos.x-12,vPos.y+12))) {bHit=true;}
		if (bHit)
		{
			vPos.y=fOld;		
			vVel.y=0;
		}

		// edge 'sensors'
		if (!IsTileSolid(GetTile(vPos.x+14,vPos.y+14))) {if (vVel.x>0) vVel.x=-vVel.x;}
		if (!IsTileSolid(GetTile(vPos.x-14,vPos.y+14))) {if (vVel.x<0) vVel.x=-vVel.x;}
		
	}



	if (nType==ENEMY_ROLLER)
	{
		vVel.y+=0.2f; // gravity;

		bool bHit=false;
		float fOld=vPos.x;
		vPos.x+=vVel.x;
		if (IsTileSolidToEnemy(GetTile(vPos.x+12,vPos.y-12))) bHit=true;
		if (IsTileSolidToEnemy(GetTile(vPos.x-12,vPos.y-12))) bHit=true;
		if (IsTileSolidToEnemy(GetTile(vPos.x+12,vPos.y+12))) bHit=true;
		if (IsTileSolidToEnemy(GetTile(vPos.x-12,vPos.y+12))) bHit=true;
		if (bHit)
		{
			vPos.x=fOld;		
			vVel.x=-vVel.x;
		}

		fOld=vPos.y;
		bHit=false;
		vPos.y+=vVel.y;
		if (IsTileSolidToEnemy(GetTile(vPos.x+12,vPos.y-12))) {bHit=true;}
		if (IsTileSolidToEnemy(GetTile(vPos.x-12,vPos.y-12))) {bHit=true;}
		if (IsTileSolidToEnemy(GetTile(vPos.x+12,vPos.y+12))) {bHit=true;}
		if (IsTileSolidToEnemy(GetTile(vPos.x-12,vPos.y+12))) {bHit=true;}
		if (bHit)
		{
			vPos.y=fOld;		
			vVel.y=0;
		}

		// handle 'teleports' (y-wrapping)
		if (GetTile(vPos.x,vPos.y)==4)
		{
			int nX=vPos.x/32;
			int nY=vPos.y/32;

			while(nY>0 && GetTileI(nX,nY-1)!=4)
			{
				nY--;
			}

			vPos.x=(nX*32)+16;
			vPos.y=(nY*32)+16;
			vVel.y=0;
		}
		
	}

};	


void Enemy::RenderLights(CGame *pGame)
{
	if (nType==ENEMY_FLOATER)
	{
		if (fGlow>0)
		{	
			LIGHT Light;
			Light.vPos=vPos;	
			Light.fR=0.2f;
			Light.fG=1.0*fGlow;
			Light.fB=0.2f;
			Light.nSubDivisions=64;
			Light.fRange=300+rand()&15;	
			Light.nTexID=TEX_POINTLIGHT;
			Light.fAngle=fAngle=gfGameTimer;
			pGame->RenderLight(&Light);
		}
	}
};	

void Enemy::Render() 
{
	if (nType==ENEMY_FLOATER)
	{
		nFrame+=1;
		if (nFrame>127) nFrame=0;
	}
	else
	{
		if (vVel.x>0)
		{
			nFrame+=1;
			if (nType==ENEMY_ROLLER) nFrame+=1;

			if (nFrame>127) nFrame=0;
		}
		else
		{
			nFrame-=1;
			if (nType==ENEMY_ROLLER) nFrame-=1;

			if (nFrame<0) nFrame=127;
		}
	}

	pSpr[nFrame>>1].Blit(vPos.x,vPos.y);
};


void Enemy::HandleCollision(CGame *pGame,Entity *pEntity)
{
	if (pEntity->nEntityType==ENTITY_BULLET)
	{
		Bullet *pBullet = (Bullet *)pEntity;
		pBullet->bAlive=false;

		nHP--;

		if (nHP<=0)
		{
			bAlive=false;
			SFX_Play(SFX_EXPLODE);
			SFX_Play(SFX_IMPACT);
			pGame->nKills++;
			Particles *pFX= new Particles(&pGame->sprExplode,32,PARTICLES_EXPLOSION,vPos.x,vPos.y,0,0);
			pGame->AddEntity(pFX);

			pGame->nScore+=150;
		}
		else
		{
			pGame->nScore+=25;
			SFX_Play(SFX_IMPACT);
			Particles *pFX= new Particles(&pGame->sprExplode,10,PARTICLES_IMPACT,vPos.x,vPos.y,vVel.x,vVel.y);
			pGame->AddEntity(pFX);
		}
	}	

	if (pEntity->nEntityType==ENTITY_PLAYER)
	{
		Particles *pFX= new Particles(&pGame->sprExplode,64,PARTICLES_EXPLOSION,pEntity->vPos.x,pEntity->vPos.y,0,0);
		pGame->AddEntity(pFX);

		pFX= new Particles(&pGame->sprExplode,32,PARTICLES_EXPLOSION,pEntity->vPos.x-10,pEntity->vPos.y,0,0);
		pGame->AddEntity(pFX);

		pFX= new Particles(&pGame->sprExplode,32,PARTICLES_EXPLOSION,pEntity->vPos.x+10,pEntity->vPos.y,0,0);
		pGame->AddEntity(pFX);

		pFX= new Particles(&pGame->sprExplode,32,PARTICLES_EXPLOSION,pEntity->vPos.x,pEntity->vPos.y-10,0,0);
		pGame->AddEntity(pFX);

		pFX= new Particles(&pGame->sprExplode,32,PARTICLES_EXPLOSION,pEntity->vPos.x,pEntity->vPos.y+10,0,0);
		pGame->AddEntity(pFX);



		pFX= new Particles(&pGame->sprExplode,128,PARTICLES_IMPACT,pEntity->vPos.x,pEntity->vPos.y,pEntity->vVel.x,pEntity->vVel.y);
		pGame->AddEntity(pFX);


		SFX_Play(SFX_EXPLODE);
		pEntity->bAlive=false;
	}
};





// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
// PARTICLES
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

Particles::Particles(Sprite *pSpr,int nCount,int nType,float fX,float fY,float fVX,float fVY)
{

	this->pSpr=pSpr;
	this->nEntityType=ENTITY_OTHER;

	this->nType=nType;

	vPos.x=fX;
	vPos.y=fY;
	fFlash=1.0f;

	for(int n=0;n<MAX_PARTICLES;n++)
	{
		if (n<nCount)
		{
			float fAng=((float)(n))/((float)(nCount))*2.0f*PI;
			p[n].bActive=true;
			p[n].vPos.x=fX;
			p[n].vPos.y=fY;
//			p[n].vVel.x=  0.01f*((float)((rand()&255)-128));
//			p[n].vVel.y=  0.01f*((float)((rand()&255)-128));
			float fRand=0.01f*((float)((rand()&255)));

			if (nType==PARTICLES_EXPLOSION) 
			{
				fRand*=0.4f;
				p[n].vVel.x=  0.5f*sin(fAng)*fRand;
				p[n].vVel.y=  0.5f*cos(fAng)*fRand;
				p[n].fLife=1.0f;
				p[n].fRotVel = 0.1f*fRand;
				p[n].fDeath=1.0f/ ((float)(25+(rand()&63)));
				p[n].fScale=0.75f;
				p[n].fRot=fRand;
				p[n].fDelay=0;
			}

			if (nType==PARTICLES_SPARKS) 
			{			
				p[n].vVel.x=  0.5f*sin(fAng)*fRand;
				p[n].vVel.y=  0.5f*cos(fAng)*fRand;
				p[n].fDelay=0.1f*fRand;
				p[n].fLife=1.0f;
				p[n].fRotVel = 0.3f*fRand;
				p[n].fDeath=1.0f/ ((float)(50+(rand()&63)));
				p[n].fScale=0.075f*fRand;
				p[n].fRot=fRand;
			}


			if (nType==PARTICLES_IMPACT) 
			{			
				p[n].vVel.x= fVX+ 1.0f*sin(fAng)*fRand;
				p[n].vVel.y= fVY+ 1.0f*cos(fAng)*fRand;
				p[n].fDelay=0.05f*fRand;
				p[n].fLife=1.0f;
				p[n].fRotVel = 0.3f*fRand;
				p[n].fDeath=1.0f/ ((float)(50+(rand()&63)));
				p[n].fScale=0.045f*fRand;
				p[n].fRot=fRand;
			}


			if (nType==PARTICLES_SPARKS2) 
			{	
				p[n].vPos.x=((float)((int)(p[n].vPos.x/32)))*32;
				p[n].vPos.y=((float)((int)(p[n].vPos.y/32)))*32;

				this->nType=PARTICLES_SPARKS;
				p[n].vPos.x+= (0.2f+(3.8f * (float)(n&7)));
				p[n].vPos.y+= (0.2f+(3.8f * (float)(n>>3)));
				p[n].vVel.x=0; // 0.5f*sin(fAng)*fRand;
				p[n].vVel.y=0;  //0.5f*cos(fAng)*fRand;
				p[n].fDelay=0.1f*fRand;
				p[n].fLife=1.0f;
				p[n].fRotVel = 0.3f*fRand;
				p[n].fDeath=1.0f/ ((float)(50+(rand()&63)));
				p[n].fScale=0.075f*fRand;
				p[n].fRot=fRand;
			}
		}
		else
		{
			p[n].bActive=false;
		}
	}
}



void Particles::Render() 
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);

	for(int n=0;n<MAX_PARTICLES;n++)	
	{
		Particle *ptl=&p[n];

		if (ptl->bActive && ptl->fDelay<=0)
		{
			glColor4f(ptl->fLife,ptl->fLife,ptl->fLife,1);
			pSpr->Blit(ptl->vPos.x,ptl->vPos.y,ptl->fRot,ptl->fScale);
		}
	}
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(1,1,1,1);
};

void Particles::RenderLights(CGame *pGame)
{
	if (nType==PARTICLES_EXPLOSION)
	{
		fFlash-=0.015f;
		if (fFlash>0)
		{
			LIGHT Light;
			Light.vPos=vPos;	
			Light.fR=1.0f*fFlash;
			Light.fG=0.7f*fFlash;
			Light.fB=0.3f*fFlash;
			Light.nSubDivisions=256;
			Light.fRange=400;	
			Light.nTexID=TEX_POINTLIGHT;
			Light.fAngle=vPos.x+gfGameTimer*2.5f;
			pGame->RenderLight(&Light);
		}
	}
	else
	{
		for(int n=0;n<MAX_PARTICLES;n++)	
		{
			Particle *ptl=&p[n];

			if (ptl->bActive && ptl->fDelay<=0)
			{
				float fScale=0.15f;
				glColor4f(ptl->fLife*fScale,ptl->fLife*fScale,ptl->fLife*fScale,1);
				pGame->sprPtlLight.Blit(ptl->vPos.x,ptl->vPos.y,ptl->fRot,ptl->fScale*2.0f);
			}
		}
	}
};

void Particles::Update(CGame *pGame)
{
	int nActive=0;
	for(int n=0;n<MAX_PARTICLES;n++)	
	{
		Particle *ptl=&p[n];

		if (ptl->bActive)
		{
			if (ptl->fDelay>0)
			{
				ptl->fDelay-=ptl->fDeath;
			}
			else
			{
				ptl->vPos.x+=ptl->vVel.x;
				if (IsTileSolid(GetTile(ptl->vPos.x,ptl->vPos.y)))
				{
					ptl->vPos.x-=ptl->vVel.x;
					ptl->vVel.x=-ptl->vVel.x/2;
				}

				ptl->vPos.y+=ptl->vVel.y;
				if (IsTileSolid(GetTile(ptl->vPos.x,ptl->vPos.y)))
				{
					ptl->vPos.y-=ptl->vVel.y;				
					ptl->vVel.y=-ptl->vVel.y/5;
				}

				if (nType!=PARTICLES_EXPLOSION)
					ptl->vVel.y+=0.05f;

 				ptl->fLife-=ptl->fDeath;
				if (ptl->fLife<0) ptl->bActive=false;		

				ptl->fRot+=ptl->fRotVel;
			}

			nActive++;
			
		}		
	}

	if (nActive==0)
	{
		this->bAlive=false;
	}
};	

