
// --------------------------------------------------------- 
//	LD48 #5 (I think)
//	
//	Theme: 'Light+Darkness'
//	By Dave Reed (bluescreen)
//
//	LD48.cpp - Init all the libs. Nothing very interesting
//             in here...  More-or-less my standard compo
//		       base code....
// --------------------------------------------------------- 

#include <stdio.h>
#include <stdlib.h>
#include <sdl.h>

#include "ld48.h"


static int  nTexNames[MAX_TEXTURES];			// OpenGL texture IDs

static bool bKeyState[NUM_KEYS];
static bool bKeyPressed[NUM_KEYS];
bool		g_bQuit=false;						
float		gfGameTimer=0;
CGame      *pGame=NULL;							// Main game object


// --------------------------------------------------------- 
// *Accurate* timer
// --------------------------------------------------------- 

int GetTicks()
{
	LARGE_INTEGER counterTicks;
	LARGE_INTEGER counterFreq;
	double        tickCount;

	QueryPerformanceCounter(&counterTicks);
	QueryPerformanceFrequency(&counterFreq);
	tickCount=((double)(counterTicks.QuadPart))*1000.0f;
	tickCount/=((double)(counterFreq.QuadPart));
	return (int)(tickCount);
}

int g_ticks=0;

// --------------------------------------------------------- 
//	Entry point/Main loop
// --------------------------------------------------------- 

int main(int argc, char **argv)
{

	int   nFlags=0;	
	bool  bSetVidMode=true;
	float t=0;

	// Init DevIL
	ilInit();
			
	// Init FMOD	
	FSOUND_Init(44100, 32, FSOUND_INIT_USEDEFAULTMIDISYNTH);

	// Init SDL
	if (SDL_Init(SDL_INIT_VIDEO| SDL_INIT_NOPARACHUTE )<0) 
	{
		fprintf(stderr,"SDL init failed: %s\n", SDL_GetError());
		exit(1);
	}
	
	SDL_WM_SetCaption("(F1 = full-screen)", NULL);

	
#ifdef FULLSCREEN
	nFlags=SDL_FULLSCREEN;
#else 
	#ifndef WINDOWED
		int r=MessageBox(NULL,"Run fullscreen?",WINDOWTITLE,MB_YESNO);
		if (r==6) nFlags=SDL_FULLSCREEN;
	#endif
#endif
	
	// Clear keypresses
	for (int n=0;n<NUM_KEYS;n++) 
	{
		bKeyState[n]=false;
	}

	pGame=new CGame;  			
	pGame->Init();
	
	
	
	// --------------------------------------------------------- 
	// Main Loop
	// --------------------------------------------------------- 

	while(!g_bQuit)
	{	
		if (bSetVidMode)
		{
			GL_SetVideoMode(WIDTH,HEIGHT,nFlags);

			ilShutDown();
			ilInit();
			iluInit();
			ilutRenderer(ILUT_OPENGL);
			ilOriginFunc(IL_ORIGIN_UPPER_LEFT);
			
			pGame->LoadTextures();

			bSetVidMode=false;
		}

		pGame->Process();
		SDL_GL_SwapBuffers();		
		SDL_Event event;


		gfGameTimer+=0.016f;

		// hacky speed limiting
		while(GetTicks()<(g_ticks+13));
		g_ticks=GetTicks();


		for (int k=0;k<NUM_KEYS;k++) 
		{
			bKeyPressed[k]=false;
		}

		while (SDL_PollEvent(&event)) 
		{

			if (event.type==SDL_QUIT) g_bQuit=TRUE;
									
			if (event.type==SDL_KEYDOWN) 		
			{
				if (event.key.keysym.sym==SDLK_F1)
				{
					if (nFlags==0) 
					{
						nFlags=SDL_FULLSCREEN;
					}
					else
					{
						nFlags=0;
					}
					bSetVidMode=true;
				}
			}

			if (event.type==SDL_KEYDOWN || event.type==SDL_KEYUP) 		
			{
				bool s=false;
				if (event.type==SDL_KEYDOWN)
				{
					s=true;				
					if (event.key.keysym.sym==SDLK_ESCAPE) g_bQuit=true; //bKeyPressed[KEY_QUIT]=true;										
					if (event.key.keysym.sym==SDLK_LEFT)   bKeyPressed[KEY_LEFT]=s;
					if (event.key.keysym.sym==SDLK_RIGHT)  bKeyPressed[KEY_RIGHT]=s;
					if (event.key.keysym.sym==SDLK_z)	   bKeyPressed[KEY_TLEFT]=s;
					if (event.key.keysym.sym==SDLK_x)	   bKeyPressed[KEY_TRIGHT]=s;
					if (event.key.keysym.sym==SDLK_UP)	   bKeyPressed[KEY_UP]=s;
					if (event.key.keysym.sym==SDLK_DOWN)   bKeyPressed[KEY_DOWN]=s;					
					if (event.key.keysym.sym==SDLK_SPACE)  bKeyPressed[KEY_FIRE]=s;
				}

				if (event.key.keysym.sym==SDLK_LEFT)   bKeyState[KEY_LEFT]=s;
				if (event.key.keysym.sym==SDLK_RIGHT)  bKeyState[KEY_RIGHT]=s;
				if (event.key.keysym.sym==SDLK_UP)	   bKeyState[KEY_UP]=s;
				if (event.key.keysym.sym==SDLK_DOWN)   bKeyState[KEY_DOWN]=s;				
				if (event.key.keysym.sym==SDLK_z)	   bKeyState[KEY_TLEFT]=s;
				if (event.key.keysym.sym==SDLK_x)	   bKeyState[KEY_TRIGHT]=s;
			}
        }		
	}

	pGame->Deinit();

	ilShutDown();
	FSOUND_Close();  
	SDL_Quit();

	return 0;
}



// --------------------------------------------------------- 
// Texture Loading
// --------------------------------------------------------- 

void GL_BindTexture(int nTex)
{
	glBindTexture(GL_TEXTURE_2D,nTexNames[nTex]);
}
 

int GL_LoadTexture(int nTex,char *szFilename)
{	
	char szFullName[512];	
	unsigned int nTmp;
	sprintf(szFullName,"data\\%s",szFilename);	

	ilGenImages( 1, &nTmp );
	ilBindImage( nTmp );	
	ilLoadImage( szFullName );
	nTexNames[nTex]=ilutGLBindMipmaps();

	GL_BindTexture(nTex);	
	return nTmp;
}



// --------------------------------------------------------- 
// Sound loader
// --------------------------------------------------------- 

FSOUND_SAMPLE *pSfx[MAX_SOUNDS];

void SFX_Load(int nId,char *szFilename)
{		
	pSfx[nId] = FSOUND_Sample_Load(FSOUND_UNMANAGED, szFilename, FSOUND_NORMAL , 0,0);
}



// --------------------------------------------------------- 
// One-liners...
// --------------------------------------------------------- 


void SFX_Play(int nId)	    {FSOUND_PlaySound(FSOUND_FREE, pSfx[nId]);}

bool KEY_State(int nKey)    {return bKeyState[nKey];}
bool KEY_Pressed(int nKey)  {return bKeyPressed[nKey];}
