

// --------------------------------------------------------- 
//	LD48 #5 (I think)
//	
//	Theme: 'Light+Darkness'
//	By Dave Reed (bluescreen)
//
//	Tilemap.cpp
// 
//  Map rendering and raycast shadows
// 
// --------------------------------------------------------- 


#include "ld48.h"


u8  gMap[MAP_WIDTH][MAP_HEIGHT];


u8 GetTile(float fX,float fY)
{
	int nX=fX/32;
	int nY=fY/32;

	if (nX<0 || nX>(MAP_WIDTH-1)) return 255;
	if (nY<0 || nY>(MAP_HEIGHT-1)) return 255;
	
	return gMap[nX][nY];		
}

u8 GetTileI(int nX,int nY)
{	
	if (nX<0 || nX>(MAP_WIDTH-1)) return 255;
	if (nY<0 || nY>(MAP_HEIGHT-1)) return 255;
	
	return gMap[nX][nY];		
}


void ClearTile(float fX,float fY)
{
	int nX=fX/32;
	int nY=fY/32;

	if (nX<0 || nX>(MAP_WIDTH-1)) return;
	if (nY<0 || nY>(MAP_HEIGHT-1)) return;

	int clr=0;
	if (GetTileI(nX+1,nY)==3) clr=3;
	if (GetTileI(nX-1,nY)==3) clr=3;
	if (GetTileI(nX,nY+1)==3) clr=3;
	if (GetTileI(nX,nY-1)==3) clr=3;

	gMap[nX][nY]=clr;

}


bool IsTileSolid(u8 n)
{	
	if ((n==1) || (n==2) || (n==5)) return true;
	return false;
}


bool IsTileSolidToEnemy(u8 n)
{	
	if ((n==1) || (n==2)) return true;
	return false;
}


float CastRay(VEC2 vOrigin, VEC2 vDir, float fMaxDist)
{
	float fDist=-1,fDistX=-1,fDistY=-1;
	bool bDone;
	VEC2 vPos;

	int nMaxSteps=(fMaxDist/32)+2;
	int nSteps,nStepLimit;

	// First, check the vertical edges
	vPos=vOrigin;
	bDone=false;

	if (vDir.x>0)
	{		
		nSteps=0; bDone=false;
		float fFrac=(32.0f-fmod(vOrigin.x,32.0f))/32.0f;		
		float fYStep = (vDir.y*32.0f)/vDir.x;

		if (abs(fYStep)>32.0f) nStepLimit=(fMaxDist/abs(fYStep))+2;
						  else nStepLimit=nMaxSteps;

		vPos.x+=(fFrac*32.0f);
		vPos.y+=(fYStep*fFrac);		
		
		while(!bDone && nSteps<nStepLimit)
		{			
			if (IsTileSolid(GetTile(vPos.x+16.0f,vPos.y)))
			{
				bDone=true;
			}		
			else
			{
				vPos.x+=32.0f;
				vPos.y+=fYStep; 
				nSteps++;
				//glBegin(GL_POINTS); glVertex2f(vPos.x,vPos.y); glEnd();
			}
		}
	}	

	if (vDir.x<0)
	{		
		nSteps=0; bDone=false;
		float fFrac=fmod(vOrigin.x,32.0f)/32.0f;		
		float fYStep = (-vDir.y*32.0f)/vDir.x;

		if (abs(fYStep)>32.0f) nStepLimit=(fMaxDist/abs(fYStep))+2;
						  else nStepLimit=nMaxSteps;

		vPos.x-=(fFrac*32.0f);
		vPos.y+=(fYStep*fFrac);		
		
		while(!bDone && nSteps<nStepLimit)
		{			
			if (IsTileSolid(GetTile(vPos.x-16.0f,vPos.y)))
			{
				bDone=true;
			}		
			else
			{
				vPos.x-=32.0f;
				vPos.y+=fYStep; 
				nSteps++;
				//glBegin(GL_POINTS); glVertex2f(vPos.x,vPos.y); glEnd();
			}
		}
	}	

	// Calc distance
	if (bDone)
	{
		float x = vPos.x-vOrigin.x;
		float y = vPos.y-vOrigin.y;
		fDistX= sqrt((x*x)+(y*y));
	}

	// Next, check the horizontal edges
	vPos=vOrigin;
	bDone=false;

	if (vDir.y>0)
	{		
		nSteps=0; bDone=false;
		float fFrac=(32.0f-fmod(vOrigin.y,32.0f))/32.0f;		
		float fXStep = (vDir.x*32.0f)/vDir.y;

		if (abs(fXStep)>32.0f) nStepLimit=(fMaxDist/abs(fXStep))+2;
						  else nStepLimit=nMaxSteps;

		vPos.y+=(fFrac*32.0f);
		vPos.x+=(fXStep*fFrac);		
		
		while(!bDone && nSteps<nStepLimit)
		{			
			if (IsTileSolid(GetTile(vPos.x,vPos.y+16.0f)))
			{
				bDone=true;
			}		
			else
			{
				vPos.y+=32.0f;
				vPos.x+=fXStep; 
				nSteps++;
				//glBegin(GL_POINTS); glVertex2f(vPos.x,vPos.y); glEnd();
			}
		}
	}	
	

	if (vDir.y<0)
	{		
		nSteps=0; bDone=false;
		float fFrac=fmod(vOrigin.y,32.0f)/32.0f;		
		float fXStep = (-vDir.x*32.0f)/vDir.y;

		if (abs(fXStep)>32.0f) nStepLimit=(fMaxDist/abs(fXStep))+2;
						  else nStepLimit=nMaxSteps;

		vPos.y-=(fFrac*32.0f);
		vPos.x+=(fXStep*fFrac);		

		while(!bDone && nSteps<nStepLimit)
		{			
			if (IsTileSolid(GetTile(vPos.x,vPos.y-16.0f)))
			{
				bDone=true;
			}		
			else
			{
				vPos.y-=32.0f;
				vPos.x+=fXStep; 
				nSteps++;
				//glBegin(GL_POINTS); glVertex2f(vPos.x,vPos.y); glEnd();
			}
		}
	}	

	// Calc distance
	if (bDone)
	{
		float x = vPos.x-vOrigin.x;
		float y = vPos.y-vOrigin.y;
		fDistY= sqrt((x*x)+(y*y));
	}


	// Pick closest hit
	if (fDistX>0)
	{
		fDist=fDistX;
	}
		
	if (fDistY>0)
	{
		if (fDistX<0) 
			fDist=fDistY;
		else
			if (fDistY<fDistX) fDist=fDistY;
	} 

	if (fDist<0 || fDist>fMaxDist)
	{
		fDist= sqrt( ((vDir.x*fMaxDist)*(vDir.x*fMaxDist)) +
		    	     ((vDir.y*fMaxDist)*(vDir.y*fMaxDist)));
	}


	return fDist;
}



// should be able to do this in HW with OpenGL, really - would boost speed!
#define TEXGEN(x,y) glTexCoord2f(0.5f+ fRecipRange*((x*rotmat[0][0])+(y*rotmat[0][1])),0.5f+fRecipRange*((x*rotmat[1][0])+(y*rotmat[1][1])));

void CGame::RenderLight(LIGHT *pLight)
{

	int   nAng;
	float fAng;
	float fDist[1024];
	float rotmat[2][2];
	

	VEC2 vLight;
	VEC2 vDir[1024];
	
	vLight.x=pLight->vPos.x;
	vLight.y=pLight->vPos.y;

	if (IsTileSolid(GetTile(vLight.x,vLight.y))) return;

	float fR=pLight->fR;
	float fG=pLight->fG;
	float fB=pLight->fB;
	float fRange=pLight->fRange;
	int nNumSubDivs=pLight->nSubDivisions;

	float fRecipRange=0.5f/(24.0f+fRange);

	rotmat[0][0]=sin(pLight->fAngle);
	rotmat[1][0]=cos(pLight->fAngle);
	rotmat[0][1]=-cos(pLight->fAngle);
	rotmat[1][1]=sin(pLight->fAngle);

	for(nAng=0; nAng<nNumSubDivs; nAng++)
	{
		
		fAng=PI * ((float)(nAng))/ ((float)(nNumSubDivs/2));

		vDir[nAng].x=sin(fAng);
		vDir[nAng].y=cos(fAng);
		
		// Cast all rays
		fDist[nAng]=CastRay(vLight,vDir[nAng],fRange);		
	}


	GL_BindTexture(pLight->nTexID);	

	glDisable(GL_CULL_FACE);

	glBegin(GL_TRIANGLE_FAN); 
	glColor3f(1.0f*fR,1.0f*fG,1.0f*fB);	
	TEXGEN(0,0);
	glVertex2f(vLight.x,vLight.y);

		
	for(nAng=0; nAng<nNumSubDivs; nAng++)
	{
		float fD=fDist[nAng];
		TEXGEN((vDir[nAng].x*fD),(vDir[nAng].y*fD))
		glVertex2f(vLight.x+(vDir[nAng].x*fD),vLight.y+(vDir[nAng].y*fD));				
	}

	float fD=fDist[0];
	TEXGEN((vDir[0].x*fD),(vDir[0].y*fD))
	glVertex2f(vLight.x+(vDir[0].x*fD),vLight.y+(vDir[0].y*fD));
	glEnd();	


	// Extrute and soften the edge to light the platforms
	glBegin(GL_TRIANGLE_STRIP); 		
	for(nAng=0; nAng<nNumSubDivs+1; nAng++)
	{
		int nAng2=nAng;
		if (nAng2>=nNumSubDivs) nAng2=0;

//		int nAng3=nAng2+1;
//		if (nAng3>=nNumSubDivs) nAng3=0;

		float fD=fDist[nAng2];
		float fD2=fD+24.0f;
//		float f2D=fDist[nAng3];
//		float f2D2=f2D+24.0f;

		glColor3f(1.0f*fR,1.0f*fG,1.0f*fB);
		TEXGEN((vDir[nAng2].x*fD),(vDir[nAng2].y*fD))
		glVertex2f(vLight.x+(vDir[nAng2].x*fD),vLight.y+(vDir[nAng2].y*fD));					
		
		glColor3f(0,0,0);
		TEXGEN((vDir[nAng2].x*fD2),(vDir[nAng2].y*fD2))
		glVertex2f(vLight.x+(vDir[nAng2].x*fD2),vLight.y+(vDir[nAng2].y*fD2));						
/*
		TEXGEN((vDir[nAng3].x*f2D2),(vDir[nAng3].y*f2D2))
		glVertex2f(vLight.x+(vDir[nAng3].x*f2D2),vLight.y+(vDir[nAng3].y*f2D2));						

		glColor3f(1.0f*fR,1.0f*fG,1.0f*fB);
		TEXGEN((vDir[nAng3].x*f2D),(vDir[nAng3].y*f2D))
		glVertex2f(vLight.x+(vDir[nAng3].x*f2D),vLight.y+(vDir[nAng3].y*f2D));					
*/
		
	}			
	glEnd();	

}



